use std::path::PathBuf;

use clap::{Arg, ArgAction, ArgMatches, Command};
use octopush_core::schema;
use octopush_infra::service_register::ServiceRegister;

pub fn execute_cmd() -> Command {
    Command::new("execute")
        .about("execute a certain action")
        .arg(
            Arg::new("action")
                .long("action")
                .short('a')
                .action(ArgAction::Set)
                .help("action path to your local octopush.yaml file")
                .long_help("action path to your local octopush.yaml file")
                .required(true),
        )
}

pub async fn execute_subcommand(args: &ArgMatches) -> eyre::Result<()> {
    let action = args
        .get_one::<String>("action")
        .ok_or(eyre::anyhow!("--action is required"))?;

    let service_register = ServiceRegister::new();

    let action_path: PathBuf = action.into();

    let schema = service_register
        .schema_parser
        .parse_file(action_path.join("octopush.yml"))
        .await?;

    match schema {
        schema::models::Schema::Action {
            name,
            select,
            actions,
        } => {
            tracing::debug!(name, "running action");

            tracing::info!("fetching repos");
            let mut repo_clones = Vec::with_capacity(select.repositories.len());
            for repo in select.repositories {
                let gp = service_register.git_provider.clone();
                repo_clones.push(tokio::spawn(async move { gp.clone_from_url(repo).await }));
            }

            let mut paths = Vec::new();
            for repo_clone in repo_clones {
                let path = repo_clone.await??;
                paths.push(path);
            }

            for path in paths {
                for action in actions.clone() {
                    service_register
                        .executor
                        .execute(path.clone(), action_path.clone(), action)
                        .await?;
                }
            }
        }
    }

    service_register.cleanup().await?;

    Ok(())
}
